function [Ca, OB, STOP]=Optics(Cb, Cc, N, nr, FR, EX)
% Ca: Coordinates of the dummy point that tell the direction of the laser ray after this control surface (after)
% OB: optical behaviour at the current control surface: 0 for refraction, 1 for reflection
% STOP: unexpected behaviour found with respect to EX, no further calculations take place for this laser ray
% Cb: Coordinates of the laser ray in the previous control surface (before)
% Cc: Coordinates of the laser ray in the current control surface (current)
% N: Vector normal to the plane at the point of intersection
% nr: ratio between refactive indices n1/n2
% FR: forced reflection (only applicable for surfaces with special coatings)
% EX: expected behaviour at the current control surface: 0 for refraction, 1 for reflection

% Rotation matrices
theta=atan2(N(1,2),N(1,1));
phi=atan((sqrt(N(1,1)^2+N(1,2)^2))/(N(1,3)));

Rz=[cos(theta)     sin(theta)   0    0;...
    -sin(theta)    cos(theta)   0    0;...
    0             0         1    0;...
    0             0         0    1];

Ry=[cos(phi)    0    -sin(phi)  0;...
    0           1        0        0;...
    sin(phi)   0    cos(phi)  0;...
    0           0        0        1];

T=[Cc(1,1);...
    Cc(1,2);...
    Cc(1,3);...
    0];

R=Ry*Rz;

Cb_NRF=R*([Cb(1,1); Cb(1,2); Cb(1,3); 1]-T);

beta_i=atan2(Cb_NRF(2,1),Cb_NRF(1,1));
alfa_i=atan((sqrt(Cb_NRF(1,1)^2+Cb_NRF(2,1)^2))/(Cb_NRF(3,1)));

if Cb_NRF(3,1)<0
    alfa_i=pi+alfa_i;
end
if Cb_NRF(3,1)==0
    alfa_i=0;
end


if alfa_i==pi/2 || alfa_i==3*pi/2 % In case the laser ray is parallel to the wall
    STOP=1;
    Ca=Cc;
    OB=2;
    
else
    
    if abs(nr*sin(alfa_i))>=1 || FR==1 %% reflection
        
        OB=1;
        Op=[-1 0 0 0; 0 -1 0 0; 0 0 1 0; 0 0 0 1];
        Ca_NRF=Op*[Cb_NRF(1:3,1); 1];
        
        Ca_NRF_l= sqrt((Ca_NRF(1,1)*Ca_NRF(1,1))+(Ca_NRF(2,1)*Ca_NRF(2,1))+(Ca_NRF(3,1)*Ca_NRF(3,1)));
        Ca_NRF=Ca_NRF(1:3,1)./Ca_NRF_l; % Vector normalization
        Ca_NRF=[Ca_NRF(1,1);Ca_NRF(2,1);Ca_NRF(3,1);1];
        
        if EX==0 % expected refraction at this control surface but instead it's reflection
            STOP=1;
        else
            STOP=0;
        end
        
    else %% refraction
        
        OB=0;
        if Cb_NRF(3,1)<0
            alfa_o=asin(nr*sin(alfa_i));
        else
            alfa_o=pi-asin(nr*sin(alfa_i));
        end
        
        beta_o=beta_i+pi;
        
        Ca_NRF=[sin(alfa_o)*cos(beta_o);  sin(alfa_o)*sin(beta_o); cos(alfa_o); 1];
        % Vector is already normalized
        
        if EX==1 % expected reflection at this control surface but instead it's refraction
            STOP=1;
        else
            STOP=0;
        end
    end
    
    Ca_calc=R\Ca_NRF+T; % Back to the borescope reference frame
    
    Ca=Ca_calc(1:3);
    
end

end

